package be.dewilde.experiments.audio.radialsoundspectrum
{
	import be.dewilde.experiments.audio.radialsoundspectrum.utils.ColorUtil;
	
	import com.bit101.components.*;
	
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.ProgressEvent;
	import flash.events.SampleDataEvent;
	import flash.filters.GlowFilter;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.media.SoundMixer;
	import flash.media.SoundTransform;
	import flash.net.URLRequest;
	import flash.sampler.NewObjectSample;
	import flash.utils.ByteArray;
	
	/**
	 * @author Kevin De Wilde
	 * Inspired by Ben Smith
	 */
	
	[SWF( backgroundColor="0x222222", width="600", height="500", frameRate="60" )]
	public class RadialSoundSpectrum extends Sprite
	{
		////////////////////////////////////////////////////////////////////////
		// Application Settings

		private var _songPath		:String = "sound.mp3";
		
		public var color			:uint = 0x1fea97; //0x1fea97		
		public var center			:Number = 150;
		public var minRadius		:int = 100;
		public	var maxRadius		:int = 110;
		public var sampleRate		:int = 1;
		public var fftMode			:Boolean = true;
		public var colorizeMode	:Boolean = false;
		
		////////////////////////////////////////////////////////////////////////
		
		private var _playing		:Boolean = false;
		private var _growing		:Boolean = true;
		private var _channel		:SoundChannel;
		private var _spectrum		:Shape;
		private var _song			:Sound;
		
		////////////////////////////////////////////////////////////////////////
		// GUI
		
		private var _guiBg:Sprite;
		private var _minRadiusLabel	:Label;
		private var _minRadiusSlider:Slider;
		private var _maxRadiusLabel	:Label;
		private var _maxRadiusSlider:Slider;
		private var _sampleLabel	:Label;
		private var _sampleSlider	:Slider;
		private var _colorLabel		:Label;
		private var _colorSlider	:Slider;
		private var _fftLabel		:Label;
		private var _fftBox			:CheckBox;
		private var _inverseLabel	:Label;
		private var _inverseBox		:CheckBox;
		private var _colorizeLabel	:Label;
		private var _colorizeBox	:CheckBox;
		private var _playButton		:PushButton;
		private var _resetButton	:PushButton;
		
		////////////////////////////////////////////////////////////////////////
		// Progress
		
		private var _progressbar	:ProgressBar;
		private var _loadingLabel	:Label;
		
		////////////////////////////////////////////////////////////////////////
		
		private var _byteArrayContainingAllBytes:ByteArray;
		private var _ch				:SoundChannel;
		private var _angIncrement	:Number;
		
		////////////////////////////////////////////////////////////////////////
		// Main
		
		public function RadialSoundSpectrum()
		{
			stage.align = StageAlign.TOP_LEFT;
			stage.scaleMode = StageScaleMode.NO_SCALE;
			
			_progressbar = new ProgressBar( this, 250, 45);
			_progressbar.width = 100;
			
			_loadingLabel = new Label(this, 250, 30, "LOADING...");
			_loadingLabel.textField.textColor = 0x1fea97;
			
			loadSound();
			
			_spectrum = new Shape();
			_spectrum.x = 150
			_spectrum.y = 100;
			addChild(_spectrum);
			
			_byteArrayContainingAllBytes = new ByteArray();
			
			_angIncrement = 360 / 256;
			
			onFrame(null);
			
			drawGUI();
		}
		
		////////////////////////////////////////////////////////////////////////
		// Sound
		
		private function loadSound():void
		{
			var soundUrlRequest:URLRequest = new URLRequest(_songPath);
			_song = new Sound(soundUrlRequest);
			_song.addEventListener( Event.COMPLETE, onSongLoadComplete );
			_song.addEventListener( ProgressEvent.PROGRESS, onLoadProgress );
		}
		
		protected function onSongLoadComplete( event:Event ):void
		{
			_progressbar.visible = false;
			_loadingLabel.visible = false;
			
			_guiBg.visible = true;
			
			_song.removeEventListener( Event.COMPLETE, onSongLoadComplete );
			_song.removeEventListener(ProgressEvent.PROGRESS, onLoadProgress );
		}
		
		////////////////////////////////////////////////////////////////////////
		// Progress Bar
		
		protected function onLoadProgress( event:ProgressEvent ):void {
			_progressbar.value = event.bytesLoaded / event.bytesTotal;
		}
		
		////////////////////////////////////////////////////////////////////////
		// Change Settings
		
		private function change(event:Event):void
		{
			minRadius = _minRadiusSlider.value;
			maxRadius = (_inverseBox.selected ? (minRadius - _maxRadiusSlider.value) : (minRadius + _maxRadiusSlider.value));
			sampleRate = _sampleSlider.value;
			fftMode = _fftBox.selected;
			colorizeMode = _colorizeBox.selected;
			
			// Color functions
			// hex -> rgb -> hsv
			// Edit hue value (1 - 359)
			// HSV -> rgb -> hex
			var rgb:Array = ColorUtil.HexToRGB(color);
			var hsv:Array = ColorUtil.RGBtoHSV(rgb[0], rgb[1], rgb[2]);
			hsv[0] = _colorSlider.value;
			rgb = ColorUtil.HSVtoRGB(hsv[0], hsv[1], hsv[2]);
			color = ColorUtil.RGBToHex(rgb[0], rgb[1], rgb[2]);
		}
		
		////////////////////////////////////////////////////////////////////////
		// Play/Pause
		
		private function toggle(event:Event):void
		{
			_playing = ! _playing;
			if (_playing)
			{
				
				_playButton.label = 'Stop';
				_channel = _song.play();
				_channel.soundTransform = new SoundTransform(1);
				addEventListener(Event.ENTER_FRAME, onFrame);
			}
			else
			{
				_playButton.label = 'Play';
				_channel.stop();
				
				removeEventListener(Event.ENTER_FRAME, onFrame);
			}
		}
		
		////////////////////////////////////////////////////////////////////////
		// Reset
		
		private function reset(event:Event):void
		{
			_minRadiusSlider.value = 100;
			_maxRadiusSlider.value = 10;
			_sampleSlider.value = 1;
			_fftBox.selected = true;
			_colorizeBox.selected = false;
			_inverseBox.selected = false;
			_colorSlider.value = 155;
			color = 0x1fea97;
			
			minRadius = _minRadiusSlider.value;
			maxRadius = (_inverseBox.selected ? (minRadius - _maxRadiusSlider.value) : (minRadius + _maxRadiusSlider.value));
			sampleRate = _sampleSlider.value;
			fftMode = _fftBox.selected;
			colorizeMode = _colorizeBox.selected;
		}
		
		////////////////////////////////////////////////////////////////////////
		// OnEnterFrame
		
		private function onFrame(event : Event):void
		{
			try
			{
				SoundMixer.computeSpectrum(_byteArrayContainingAllBytes, fftMode, sampleRate);
			}
			catch (e:Error)
			{
				throw new Error('Currently you have another flash item open that is playing sounds.  Please close other windows to continue');
			}
			
			var leftChannelArrayOnly : Vector.<Number> = new Vector.<Number>();
			_byteArrayContainingAllBytes.position = 0;
			var index:int = 0;
			var leftChannel:int = 250; //256 samples for left channel  // 250 for smooth transition
			
			while (index<leftChannel)
			{
				leftChannelArrayOnly[index] = Number(_byteArrayContainingAllBytes.readFloat());
				index++;
			}
			
			
			var l:int = leftChannelArrayOnly.length;
			var firstPoint:Boolean = false;
			var endPtX:Number;
			var endPtY:Number;
			_spectrum.graphics.clear();
			_spectrum.graphics.lineStyle(1,color);
			_spectrum.filters = [new GlowFilter(color,0.6, 10, 10)];
			
			while (--l>=0)
			{
				var ang:Number= ((l*_angIncrement)*Math.PI)/180;
				
				var rasd:Number=minRadius + int((leftChannelArrayOnly[l] * ((maxRadius - minRadius))));
				
				var ypt:Number = center + Math.sin(ang) * rasd;// (leftChannelArrayOnly[l]*100);
				var xpt:Number = center + Math.cos(ang) * rasd;//(leftChannelArrayOnly[l]*100)
				if (! firstPoint)
				{
					firstPoint = true;
					endPtX = xpt;
					endPtY = ypt;
					_spectrum.graphics.moveTo(xpt,ypt);
				}
				
				_spectrum.graphics.lineTo(xpt,ypt);
			}
			
			//ColorizeMode
			if (colorizeMode)
			{
				var rgb:Array = ColorUtil.HexToRGB(color);
				var hsv:Array = ColorUtil.RGBtoHSV(rgb[0], rgb[1], rgb[2]);
				if (hsv[0] < 359)
				{
					hsv[0] += 1;
					_colorSlider.value += 1;
				}
				else
				{
					hsv[0] = 0;
					_colorSlider.value = 10;
				}
				rgb = ColorUtil.HSVtoRGB(hsv[0], hsv[1], hsv[2]);
				color = ColorUtil.RGBToHex(rgb[0], rgb[1], rgb[2]);
			}
			
			_spectrum.graphics.lineTo(endPtX,endPtY);
			_spectrum.graphics.endFill();
		}
		
		///////////////////////////////////////////////////////////////////////
		// GUI
		
		private function drawGUI():void
		{
			_guiBg = new Sprite();
			_guiBg.visible = false;
			_guiBg.graphics.beginFill( 0xFFFFFF, .15 );
			_guiBg.graphics.drawRect( 0, 0, 460, 100);
			_guiBg.graphics.endFill();
			_guiBg.x = 70;
			_guiBg.y = 5;
			
			_minRadiusLabel = new Label(_guiBg, 10, 5, "radius");
			_minRadiusLabel.textField.textColor = 0x1fea97;
			
			_minRadiusSlider = new Slider("horizontal", _guiBg, 100, 10, change);
			_minRadiusSlider.value = 100;
			_minRadiusSlider.minimum = 0;
			_minRadiusSlider.maximum =100;
			
			_maxRadiusLabel = new Label(_guiBg, 10, 30, "maximum radius");
			_maxRadiusLabel.textField.textColor = 0x1fea97;
			
			_maxRadiusSlider = new Slider("horizontal", _guiBg, 100, 35, change);
			_maxRadiusSlider.minimum = 10;
			_maxRadiusSlider.maximum = 100;
			_maxRadiusSlider.value = 10;
			
			_sampleLabel = new Label(_guiBg, 10, 55, "Sample rate");
			_sampleLabel.textField.textColor = 0x1fea97;
			
			_sampleSlider = new Slider("horizontal", _guiBg, 100, 60, change);
			_sampleSlider.minimum = 1;
			_sampleSlider.maximum = 100;
			_sampleSlider.value = 1;
			
			_colorLabel = new Label(_guiBg, 10, 80, "Color");
			_colorLabel.textField.textColor = 0x1fea97;
			
			_colorSlider = new Slider("horizontal", _guiBg, 100, 85, change);
			_colorSlider.minimum = 1;
			_colorSlider.maximum = 359;
			_colorSlider.value = 155;
			
			_fftLabel = new Label(_guiBg, 270, 5, "FFT Mode");
			_fftLabel.textField.textColor = 0x1fea97;
			
			_fftBox = new CheckBox(_guiBg, 250, 10, null, change);
			_fftBox.selected = true;
			
			_inverseLabel = new Label(_guiBg, 270, 30, "Inverse");
			_inverseLabel.textField.textColor = 0x1fea97;
			
			_inverseBox = new CheckBox(_guiBg, 250, 35, null, change);
			_inverseBox.selected = false;
			
			_colorizeLabel = new Label(_guiBg, 270, 55, "Colorize");
			_colorizeLabel.textField.textColor = 0x1fea97;
			
			_colorizeBox = new CheckBox(_guiBg, 250, 60, null, change);
			_colorizeBox.selected = false;
			
			_playButton = new PushButton(_guiBg, 350, 10, "Play", toggle);
			
			_resetButton = new PushButton(_guiBg, 350, 40, "Reset", reset);
			
			addChildAt( _guiBg, 0 );
		}
		
	}
}